<?php

/**
 * Creates views in code.
 *
 * @code
 * $my_block = CINC::init('View')->machine_name('a_view')->read();
 * @endcode
 */
class CINC_View extends CINC implements CINC_CRUD {

  public static $dependencies = array('views');
  public $config_machine_name = 'name';
  public $cinc_yaml_prefix = 'view';

  function __construct() {

    $this->config = array(
      'db_table' => 'views_view',
      'table' => 'views_view',
      'type' => 'Normal',
      'export_type' => 1,
      'base_table' => 'node',
      'base_field' => 'nid',
      'name' => '',
      'vid' => 'new',
      'description' => '',
      'tag' => 'default',
      'human_name' => '',
      'core' => 7,
      'display' => array(),
    );

    $this->add_default_display();

  }

  /**
   * Saves view to the database.
   */
  public function create() {

    $view = $this->get_view_object();
    $view->save();

    return $this;
  }

  /**
   * Tests if view exists.
   */
  public function exists() {

    $view = views_get_view($this->get('name'));
    if ($view) {
      return TRUE;
    }
    return FALSE;

  }

  /**
   * Loads view from the database.
   */
  public function read() {
    $view = views_get_view($this->get('name'));
    $this->set_from_view_object($view);
    return $this;
  }

  /**
   * Loads list of machine names.
   */
  static public function read_list() {

    $list = array();
    return $list;

  }

  /**
   * Saves view to the database.
   */
  public function update() {
    $this->create();
    return $this;
  }

  /**
   * Removes view from the database.
   */
  public function delete() {
    $view = $this->get_view_object();
    views_delete_view($view);
    $this->config = array();
    return $this;
  }

  public function set_row_style($type, $display_id = 'default') {

    switch ($type) {

      case 'node':
        $this
          ->set('display.' . $display_id . '.display_options.row_plugin', 'node')
          ->set('display.' . $display_id . '.display_options.row_options.build_mode', 'full');
        break;

    }

    return $this;
  }

  public function set_view_mode($view_mode, $display_id = 'default') {
    $this->set('display.' . $display_id . '.display_options.row_options.build_mode', $view_mode);
    return $this;
  }

  public function add_default_display() {
    $display = new views_display;
    $display->options('default', 'default', 'Master');
    $this->config['display']['default'] = (array) $display;

    return $this;
  }

  public function add_block_display($id = 'block', $title = 'Block') {
    $display = new views_display;
    $display->options('block', $id, $title);
    $this->config['display'][$id] = (array) $display;

    return $this;
  }

  public function add_footer($type = 'area', $display_id = 'default', $details = array()) {
    $display = cinc_object_from_array($this->config['display'][$display_id], 'views_display');
    if (!isset($display->display_options['footer'])) {
      $display->display_options['footer'] = array();
    }

    switch ($type) {

      case 'area':
        $display->display_options['footer']['area'] = array(
          'id' => 'area',
          'table' => 'views',
          'field' => 'area',
          'label' => 'Footer',
          'content' => '',
          'format' => 'full_html',
        ) + $details;
        break;

    }

    $this->config['display'][$display_id] = (array) $display;

    return $this;  }

  public function add_sort($type, $display_id = 'default', $details = array()) {
    $display = cinc_object_from_array($this->config['display'][$display_id], 'views_display');
    if (!isset($display->display_options['sorts'])) {
      $display->display_options['sorts'] = array();
    }

    switch ($type) {

      case 'random':
        $display->display_options['sorts']['random'] = array(
          'id' => 'random',
          'table' => 'views',
          'field' => 'random',
          'relationship' => 'none',
          'group_type' => 'group',
          'ui_name' => '',
          'order' => 'ASC',
          'exposed' => FALSE,
        ) + $details;
        break;

    }

    $this->config['display'][$display_id] = (array) $display;

    return $this;
  }

  public function add_filter($type, $display_id = 'default', $details = array()) {
    $display = cinc_object_from_array($this->config['display'][$display_id], 'views_display');
    if (!isset($display->display_options['filters'])) {
      $display->display_options['filters'] = array();
    }

    switch ($type) {

      case 'published':
        $display->display_options['filters']['status'] = array(
          'value' => 1,
          'table' => 'node',
          'field' => 'status',
          'id' => 'status',
          'group' => 1,
        )  + $details;
        break;
      case 'node_type':
        $display->display_options['filters']['type'] = array(
          'id' => 'type',
          'table' => 'node',
          'field' => 'type',
        ) + $details;
        break;
      case 'taxonomy_term':
        $display->display_options['filters']['tid'] = array(
          'id' => 'tid',
          'table' => 'taxonomy_index',
          'field' => 'tid',
        ) + $details;
        break;

    }

    $this->config['display'][$display_id] = (array) $display;

    return $this;
  }

  /**
   * Gets view object based on config.
   */
  public function get_view_object() {
    $view = cinc_object_from_array($this->config, 'view');

    foreach ($view->display as $key => $display_array) {
      $view->display[$key] = cinc_object_from_array($display_array, 'views_display');
    }

    return $view;
  }

  /**
   * Sets config based on view object.
   */
  public function set_from_view_object($view) {
    $this->config = (array) $view;
    foreach ($this->config['display'] as $key => $display) {
      $this->config['display'][$key] = (array) $display;
    }
  }

}
